@@ -6,6 +6,7 @@ QSI workflow
 """
 import warnings
 import os
+import re
 import os.path as op
 from pathlib import Path
 import logging
@@ -83,6 +84,10 @@ def get_parser():
         'identifier (the sub- prefix can be removed)')
     g_bids.add_argument('--acquisition_type', '--acquisition_type', action='store',
                         help='select a specific acquisition type to be processed')
+    g_bids.add_argument('--bids-database-dir', '--bids_database_dir',
+                        help="path to a saved BIDS database directory",
+                        type=Path,
+                        action='store')
 
     # arguments for reconstructing QSI data
     g_ireports = parser.add_argument_group('Options for interactive report outputs')
@@ -192,16 +197,23 @@ def get_parser():
     g_conf.add_argument(
         '--dwi_denoise_window', '--dwi-denoise-window',
         action='store',
-        type=int,
-        default=5,
-        help='window size in voxels for ``dwidenoise``. Must be odd (default: 5). '
-             'If 0, ``dwidwenoise`` will not be run')
+        default='auto',
+        help='window size in voxels for image-based denoising, integer or "auto".'
+             'If "auto", 5 will be used for dwidenoise and auto-configured for '
+             'patch2self based on the number of b>0 images.')
+    g_conf.add_argument(
+        '--denoise-method', '--denoise_method',
+        action='store',
+        choices=['dwidenoise', 'patch2self', 'none'],
+        default='dwidenoise',
+        help='Image-based denoising method. Either "dwidenoise" (MRtrix), '
+             '"patch2self" (DIPY) or none. (default: dwidenoise)')
     g_conf.add_argument(
         '--unringing-method', '--unringing_method',
         action='store',
         choices=['none', 'mrdegibbs'],
         help='Method for Gibbs-ringing removal.\n - none: no action\n - mrdegibbs: '
-             'use mrdegibbs from mrtrix3')
+             'use mrdegibbs from mrtrix3 (default: none).')
     g_conf.add_argument(
         '--dwi-no-biascorr', '--dwi_no_biascorr',
         action='store_true',
@@ -210,22 +222,11 @@ def get_parser():
         '--no-b0-harmonization', '--no_b0_harmonization',
         action='store_true',
         help='skip re-scaling dwi scans to have matching b=0 intensities')
-    g_conf.add_argument(
-        '--denoise-before-combining', '--denoise_before_combining',
-        action='store_true',
-        help='[DEPRECATED] run ``dwidenoise`` before combining dwis. Requires '
-             '``--combine-all-dwis``')
     g_conf.add_argument(
         '--denoise-after-combining', '--denoise_after_combining',
         action='store_true',
         help='run ``dwidenoise`` after combining dwis. Requires '
              '``--combine-all-dwis``')
-    g_conf.add_argument(
-        '--combine_all_dwis', '--combine-all-dwis',
-        action='store_true',
-        default=False,
-        help='[DEPRECATED] combine dwis from across multiple runs for motion correction '
-        'and reconstruction.')
     g_conf.add_argument(
         '--separate_all_dwis', '--separate-all-dwis',
         action='store_true',
@@ -241,8 +242,7 @@ def get_parser():
         ' - average: if a whole sequence was duplicated in both PE\n'
         '            directions, average the corrected images of the same\n'
         '            q-space coordinate\n'
-        ' - none: Default. Keep distorted groups separate'
-    )
+        ' - none: Default. Keep distorted groups separate')
     g_conf.add_argument(
         '--write-local-bvecs', '--write_local_bvecs',
         action='store_true',
@@ -360,11 +360,6 @@ def get_parser():
         action='store_true',
         help='do not use a random seed for skull-stripping - will ensure '
         'run-to-run replicability when used with --omp-nthreads 1')
-    g_ants.add_argument(
-        '--force-spatial-normalization', '--force_spatial_normalization',
-        action='store_true',
-        help='[DEPRECATED] ensures that spatial normalization is run, '
-        'Useful if you plan to warp atlases into subject space.')
     g_ants.add_argument(
         '--skip-t1-based-spatial-normalization', '--skip_t1_based_spatial_normalization',
         action='store_true',
@@ -736,7 +731,7 @@ def build_qsiprep_workflow(opts, retval):
 
     bids_dir = opts.bids_dir.resolve()
     output_dir = opts.output_dir.resolve()
-    work_dir = opts.work_dir.resolve()
+    work_dir = Path(opts.work_dir or 'work')  # Set work/ as default
 
     retval['return_code'] = 1
     retval['workflow'] = None
@@ -756,8 +751,22 @@ def build_qsiprep_workflow(opts, retval):
     run_uuid = '%s_%s' % (strftime('%Y%m%d-%H%M%S'), uuid.uuid4())
     retval['run_uuid'] = run_uuid
 
+    _db_path = opts.bids_database_dir or (
+        work_dir / run_uuid / "bids_db")
+    _db_path.mkdir(exist_ok=True, parents=True)
+
     # First check that bids_dir looks like a BIDS folder
-    layout = BIDSLayout(str(bids_dir), validate=False)
+    layout = BIDSLayout(
+                str(bids_dir),
+                validate=False,
+                database_path=_db_path,
+                reset_database=opts.bids_database_dir is None,
+                ignore=(
+                    "code",
+                    "stimuli",
+                    "sourcedata",
+                    "models",
+                    re.compile(r"^\.")))
     subject_list = collect_participants(
         layout, participant_label=opts.participant_label)
     retval['subject_list'] = subject_list
@@ -769,10 +778,7 @@ def build_qsiprep_workflow(opts, retval):
                        "Spatial normalization should be done during reconstruction.")
         output_spaces = ["T1w"]
 
-    # Deprecated --force-spatial-normalization
     force_spatial_normalization = not opts.skip_t1_based_spatial_normalization
-    if opts.force_spatial_normalization:
-        logger.warning("[DEPRECATED] --force-spatial-normalization is now default")
     if not force_spatial_normalization and (opts.use_syn_sdc or opts.force_syn):
         msg = [
             'SyN SDC correction requires T1 to MNI registration.',
@@ -781,14 +787,6 @@ def build_qsiprep_workflow(opts, retval):
         force_spatial_normalization = True
         logger.warning(' '.join(msg))
 
-    # deprecated --combine-all-dwis
-    if opts.combine_all_dwis:
-        logger.warning("[DEPRECATED] DWIs are combined by default.")
-
-    # deprecated --denoise-before-combining
-    if opts.denoise_before_combining:
-        logger.warning("[DEPRECATED] DWIs are denoised before combining by default.")
-
     # Load base plugin_settings from file if --use-plugin
     if opts.use_plugin is not None:
         from yaml import load as loadyml
@@ -895,6 +893,7 @@ def build_qsiprep_workflow(opts, retval):
         anat_only=opts.anat_only,
         longitudinal=opts.longitudinal,
         b0_threshold=opts.b0_threshold,
+        denoise_method=opts.denoise_method,
         combine_all_dwis=not opts.separate_all_dwis,
         distortion_group_merge=opts.distortion_group_merge,
         dwi_denoise_window=opts.dwi_denoise_window,
@@ -978,7 +977,7 @@ def build_recon_workflow(opts, retval):
     """
     from subprocess import check_call, CalledProcessError, TimeoutExpired
     from pkg_resources import resource_filename as pkgrf
-
+    from bids import BIDSLayout
     from nipype import logging, config as ncfg
     from ..__about__ import __version__
     from ..workflows.recon import init_qsirecon_wf
@@ -995,11 +994,31 @@ def build_recon_workflow(opts, retval):
 
     # Set up some instrumental utilities
     run_uuid = '%s_%s' % (strftime('%Y%m%d-%H%M%S'), uuid.uuid4())
+    # Set up directories
+    output_dir = op.abspath(opts.output_dir)
+    log_dir = op.join(output_dir, 'qsirecon', 'logs')
+    work_dir = Path(opts.work_dir or 'work')  # Set work/ as default
+
+    _db_path = opts.bids_database_dir or (
+        work_dir / run_uuid / "bids_db")
+    _db_path.mkdir(exist_ok=True, parents=True)
 
     # First check that bids_dir looks like a BIDS folder
-    bids_dir = os.path.abspath(opts.bids_dir)
+    bids_dir = opts.bids_dir.resolve()
+    layout = BIDSLayout(
+                str(bids_dir),
+                validate=False,
+                database_path=_db_path,
+                reset_database=opts.bids_database_dir is None,
+                ignore=(
+                    "code",
+                    "stimuli",
+                    "sourcedata",
+                    "models",
+                    re.compile(r"^\.")))
     subject_list = collect_participants(
-        bids_dir, participant_label=opts.participant_label, bids_validate=False)
+        layout, participant_label=opts.participant_label)
+    retval['subject_list'] = subject_list
 
     # Load base plugin_settings from file if --use-plugin
     if opts.use_plugin is not None:
@@ -1040,11 +1059,6 @@ def build_recon_workflow(opts, retval):
             'Per-process threads (--omp-nthreads=%d) exceed total '
             'threads (--nthreads/--n_cpus=%d)', omp_nthreads, nthreads)
 
-    # Set up directories
-    output_dir = op.abspath(opts.output_dir)
-    log_dir = op.join(output_dir, 'qsirecon', 'logs')
-    work_dir = op.abspath(opts.work_dir or 'work')  # Set work/ as default
-
     # Check and create output and working directories
     os.makedirs(output_dir, exist_ok=True)
     os.makedirs(log_dir, exist_ok=True)
@@ -1103,7 +1117,6 @@ def build_recon_workflow(opts, retval):
         recon_spec=opts.recon_spec,
         low_mem=opts.low_mem,
         omp_nthreads=omp_nthreads,
-        bids_dir=bids_dir,
         sloppy=opts.sloppy
     )
     retval['return_code'] = 0
