@@ -19,12 +19,15 @@ from dipy.segment.mask import median_otsu
 from dipy.core.sphere import HemiSphere
 from dipy.core.gradients import gradient_table
 from dipy.reconst import mapmri, dti
+from nilearn.image import load_img
 from nipype import logging
 from nipype.utils.filemanip import fname_presuffix
 from nipype.interfaces.base import (
     traits, TraitedSpec, BaseInterfaceInputSpec, File, SimpleInterface, isdefined
 )
-
+from .patch2self import patch2self
+from .denoise import (SeriesPreprocReport, SeriesPreprocReportInputSpec,
+                      SeriesPreprocReportOutputSpec)
 from .converters import get_dsi_studio_ODF_geometry, amplitudes_to_fibgz, amplitudes_to_sh_mif
 from ..utils.brainsuite_shore import BrainSuiteShoreModel, brainsuite_shore_basis
 from ..interfaces.mrtrix import _convert_fsl_to_mrtrix
@@ -80,12 +83,118 @@ class MedianOtsu(SimpleInterface):
         masked_img = nb.Nifti1Image(masked_data, b0_img.affine, b0_img.header)
         masked_img.to_filename(self._results['masked_input'])
 
-        mask_img = nb.Nifti1Image(data_mask.astype('f8'), b0_img.affine, b0_img.header)
+        mask_img = nb.Nifti1Image(data_mask.astype('f8'), b0_img.affine,
+                                  b0_img.header)
         mask_img.to_filename(self._results['out_mask'])
 
         return runtime
 
 
+class Patch2SelfInputSpec(SeriesPreprocReportInputSpec):
+    in_file = File(exists=True, mandatory=True,
+                   desc="4D diffusion MRI data file")
+    patch_radius = traits.Either(traits.Int(0), traits.Str('auto'),
+                                 desc='patch radius in voxels.')
+    bval_file = File(exists=True, mandatory=True,
+                     desc="bval file containing b-values")
+    model = traits.Str('ridge', usedefault=True,
+                       desc='Regression model for Patch2Self')
+    alpha = traits.Float(1., usedefault=True,
+                         desc='Regularization parameter for Ridge and Lasso')
+    b0_threshold = traits.Float(50., usedefault=True,
+                                desc='Threshold to segregate b0s')
+    mask = File(desc='mask image (unused)')
+    b0_denoising = traits.Bool(True, usedefault=True,
+                               desc='denoise the b=0 images too')
+    clip_negative_vals = traits.Bool(False, usedefault=True,
+                                     desc='Sets negative values after denoising to 0')
+    shift_intensity = traits.Bool(True, usedefault=True,
+                                  desc='Shifts the distribution of intensities per '
+                                  'volume to give non-negative values')
+    out_report = File('patch2self_report.svg', usedefault=True,
+                      desc='filename for the visual report')
+
+
+class Patch2SelfOutputSpec(SeriesPreprocReportOutputSpec):
+    out_file = File(exists=True,
+                    desc='Denoised version of the input image')
+    noise_image = File(exists=True,
+                       desc='Residuals depicting suppressed noise')
+
+
+class Patch2Self(SeriesPreprocReport, SimpleInterface):
+    input_spec = Patch2SelfInputSpec
+    output_spec = Patch2SelfOutputSpec
+
+    def _run_interface(self, runtime):
+
+        in_file = self.inputs.in_file
+        bval_file = self.inputs.bval_file
+        denoised_file = fname_presuffix(
+            in_file, suffix='_denoised_patch2self', newpath=runtime.cwd)
+        noise_file = fname_presuffix(
+            in_file, suffix='_denoised_residuals_patch2self',
+            newpath=runtime.cwd)
+        noisy_img = nb.load(in_file)
+        noisy_arr = noisy_img.get_fdata()
+        bvals = np.loadtxt(bval_file)
+
+        # Determine the patch radius
+        num_non_b0 = (bvals > self.inputs.b0_threshold).sum()
+        very_few_directions = num_non_b0 < 20
+        few_directions = num_non_b0 < 50
+        if self.inputs.patch_radius == 'auto':
+            if very_few_directions:
+                patch_radius = [3, 3, 3]
+            elif few_directions:
+                patch_radius = [1, 1, 1]
+            else:
+                patch_radius = [0, 0, 0]
+        else:
+            patch_radius = [self.inputs.patch_radius] * 3
+            if self.inputs.patch_radius > 3 and not very_few_directions:
+                LOGGER.info("a very large patch radius is not necessary when more than "
+                            "20 gradient directions have been sampled.")
+            elif self.inputs.patch_radius > 1 and not few_directions:
+                LOGGER.info("a large patch radius is not necessary when more than "
+                            "50 gradient directions have been sampled.")
+            elif self.inputs.patch_radius == 0 and few_directions:
+                LOGGER.warning("When < 50 gradient directions are available, it is "
+                               "recommended to increase patch_radius to > 0.")
+
+        denoised_arr, noise_residuals = \
+            patch2self(noisy_arr, bvals,
+                       model=self.inputs.model,
+                       alpha=self.inputs.alpha,
+                       patch_radius=patch_radius,
+                       b0_threshold=self.inputs.b0_threshold,
+                       verbose=True,
+                       b0_denoising=self.inputs.b0_denoising,
+                       clip_negative_vals=self.inputs.clip_negative_vals,
+                       shift_intensity=self.inputs.shift_intensity
+                       )
+
+        # Back to nifti
+        denoised_img = nb.Nifti1Image(denoised_arr, noisy_img.affine,
+                                      noisy_img.header)
+        p2s_residuals = nb.Nifti1Image(noise_residuals, noisy_img.affine,
+                                       noisy_img.header)
+        denoised_img.to_filename(denoised_file)
+        p2s_residuals.to_filename(noise_file)
+        self._results['out_file'] = denoised_file
+        self._results['noise_image'] = noise_file
+        return runtime
+
+    def _get_plotting_images(self):
+        input_dwi = load_img(self.inputs.in_file)
+        outputs = self._list_outputs()
+        ref_name = outputs.get('out_file')
+        denoised_nii = load_img(ref_name)
+        noise_name = outputs['noise_image']
+        noisenii = load_img(noise_name)
+        return input_dwi, denoised_nii, noisenii
+
+
 class HistEQInputSpec(BaseInterfaceInputSpec):
     in_file = File(exists=True, mandatory=True,
                    desc='File to equalize')
@@ -112,7 +221,8 @@ class HistEQ(SimpleInterface):
         data_voxels = uneq_data[bool_mask]
 
         # Do a clip on 2 to 98th percentile
-        bottom_2, top_98 = np.percentile(data_voxels, np.array([1, 99]), axis=None)
+        bottom_2, top_98 = np.percentile(data_voxels, np.array([1, 99]),
+                                         axis=None)
         clipped_b0 = np.clip(data_voxels, 0, top_98)
         eq_data = histeq(clipped_b0, num_bins=512)
         output = np.zeros_like(mask.get_data())
@@ -171,7 +281,11 @@ class DipyReconInterface(SimpleInterface):
         if not isdefined(self.inputs.mask_file):
             dwi_data = amplitudes_img.get_data()
             LOGGER.warning("Creating an Otsu mask, check that the whole brain is covered.")
-            _, mask_array = median_otsu(dwi_data[..., gtab.b0s_mask], 3, 2)
+            _, mask_array = median_otsu(dwi_data,
+                                        vol_idx=gtab.b0s_mask,
+                                        median_radius=3,
+                                        numpass=2)
+
             # Needed for synthetic data
             mask_array = mask_array * (dwi_data.sum(3) > 0)
             mask_img = nb.Nifti1Image(mask_array.astype(np.float32), amplitudes_img.affine,
