@@ -26,7 +26,7 @@ from ..engine import Workflow
 from ..interfaces import (BIDSDataGrabber, BIDSInfo, BIDSFreeSurferDir,
                           SubjectSummary, AboutSummary, DerivativesDataSink)
 from ..utils.bids import collect_data
-from ..utils.misc import fix_multi_T1w_source_name
+from ..utils.misc import fix_multi_T1w_source_name, fix_multi_source_name
 from ..utils.grouping import group_dwi_scans
 from ..__about__ import __version__
 
@@ -44,13 +44,13 @@ LOGGER = logging.getLogger('nipype.workflow')
 def init_qsiprep_wf(
         subject_list, run_uuid, work_dir, output_dir, bids_dir, ignore, debug, low_mem, anat_only,
         dwi_only, longitudinal, b0_threshold, hires, denoise_before_combining, dwi_denoise_window,
-        unringing_method, dwi_no_biascorr, no_b0_harmonization, output_resolution, infant_mode,
-        combine_all_dwis, distortion_group_merge, omp_nthreads, force_spatial_normalization,
-        skull_strip_template, skull_strip_fixed_seed, freesurfer, hmc_model,
-        impute_slice_threshold, hmc_transform, shoreline_iters, eddy_config, write_local_bvecs,
-        output_spaces, template, motion_corr_to, b0_to_t1w_transform, intramodal_template_iters,
-        intramodal_template_transform, prefer_dedicated_fmaps, fmap_bspline, fmap_demean, use_syn,
-        force_syn):
+        denoise_method, unringing_method, dwi_no_biascorr, no_b0_harmonization, output_resolution,
+        infant_mode, combine_all_dwis, distortion_group_merge, omp_nthreads,
+        force_spatial_normalization, skull_strip_template, skull_strip_fixed_seed, freesurfer,
+        hmc_model, impute_slice_threshold, hmc_transform, shoreline_iters, eddy_config,
+        write_local_bvecs, output_spaces, template, motion_corr_to, b0_to_t1w_transform,
+        intramodal_template_iters, intramodal_template_transform, prefer_dedicated_fmaps,
+        fmap_bspline, fmap_demean, use_syn, force_syn):
     """
     This workflow organizes the execution of qsiprep, with a sub-workflow for
     each subject.
@@ -78,6 +78,7 @@ def init_qsiprep_wf(
                               hires=False,
                               denoise_before_combining=True,
                               dwi_denoise_window=7,
+                              denoise_method='patch2self',
                               unringing_method='mrdegibbs',
                               dwi_no_biascorr=False,
                               no_b0_harmonization=False,
@@ -138,8 +139,10 @@ def init_qsiprep_wf(
         b0_threshold : int
             Images with b-values less than this value will be treated as a b=0 image.
         dwi_denoise_window : int
-            window size in voxels for ``dwidenoise``. Must be odd. If 0, '
-            '``dwidwenoise`` will not be run'
+            window size in voxels for image-based denoising. Must be odd. If 0, '
+            'denoising will not be run'
+        denoise_method : str
+            Either 'dwidenoise', 'patch2self' or 'none'
         unringing_method : str
             algorithm to use for removing Gibbs ringing. Options: none, mrdegibbs
         dwi_no_biascorr : bool
@@ -235,6 +238,7 @@ def init_qsiprep_wf(
             output_resolution=output_resolution,
             denoise_before_combining=denoise_before_combining,
             dwi_denoise_window=dwi_denoise_window,
+            denoise_method=denoise_method,
             unringing_method=unringing_method,
             dwi_no_biascorr=dwi_no_biascorr,
             no_b0_harmonization=no_b0_harmonization,
@@ -284,8 +288,8 @@ def init_qsiprep_wf(
 def init_single_subject_wf(
         subject_id, name, reportlets_dir, output_dir, bids_dir, ignore, debug, write_local_bvecs,
         low_mem, dwi_only, anat_only, longitudinal, b0_threshold, denoise_before_combining,
-        dwi_denoise_window, unringing_method, dwi_no_biascorr, no_b0_harmonization, infant_mode,
-        combine_all_dwis, distortion_group_merge, omp_nthreads, skull_strip_template,
+        dwi_denoise_window, denoise_method, unringing_method, dwi_no_biascorr, no_b0_harmonization,
+        infant_mode, combine_all_dwis, distortion_group_merge, omp_nthreads, skull_strip_template,
         force_spatial_normalization, skull_strip_fixed_seed, freesurfer, hires, output_spaces,
         template, output_resolution, prefer_dedicated_fmaps, motion_corr_to, b0_to_t1w_transform,
         intramodal_template_iters, intramodal_template_transform, hmc_model, hmc_transform,
@@ -319,6 +323,7 @@ def init_single_subject_wf(
             output_resolution=1.25,
             denoise_before_combining=True,
             dwi_denoise_window=7,
+            denoise_method='patch2self',
             unringing_method='mrdegibbs',
             dwi_no_biascorr=False,
             no_b0_harmonization=False,
@@ -375,8 +380,10 @@ def init_single_subject_wf(
         b0_threshold : int
             Images with b-values less than this value will be treated as a b=0 image.
         dwi_denoise_window : int
-            window size in voxels for ``dwidenoise``. Must be odd. If 0, '
-            '``dwidwenoise`` will not be run'
+            window size in voxels for image-based denoising. Must be odd. If 0, '
+            'denoising will not be run'
+        denoise_method : str
+            Either 'dwidenoise', 'patch2self' or 'none'
         unringing_method : str
             algorithm to use for removing Gibbs ringing. Options: none, mrdegibbs
         dwi_no_biascorr : bool
@@ -559,10 +566,11 @@ to workflows in *QSIPrep*'s documentation]\
         output_dir=output_dir,
         num_t1w=len(subject_data['t1w']))
 
+    info_modality = "dwi" if dwi_only else "t1w"
     workflow.connect([
         (inputnode, anat_preproc_wf, [('subjects_dir',
                                        'inputnode.subjects_dir')]),
-        (bidssrc, bids_info, [(('t1w', fix_multi_T1w_source_name),
+        (bidssrc, bids_info, [((info_modality, fix_multi_source_name, dwi_only),
                                'in_file')]),
         (inputnode, summary, [('subjects_dir', 'subjects_dir')]),
         (bidssrc, summary, [('t1w', 't1w'), ('t2w', 't2w')]),
@@ -572,12 +580,12 @@ to workflows in *QSIPrep*'s documentation]\
                                     ('roi', 'inputnode.roi'),
                                     ('flair', 'inputnode.flair')]),
         (summary, anat_preproc_wf, [('subject_id', 'inputnode.subject_id')]),
-        (bidssrc, ds_report_summary, [(('t1w', fix_multi_T1w_source_name),
+        (bidssrc, ds_report_summary, [((info_modality, fix_multi_source_name, dwi_only),
                                        'source_file')]),
         (summary, ds_report_summary, [('out_report', 'in_file')]),
-        (bidssrc, ds_report_about, [(('t1w', fix_multi_T1w_source_name),
+        (bidssrc, ds_report_about, [((info_modality, fix_multi_source_name, dwi_only),
                                      'source_file')]),
-        (about, ds_report_about, [('out_report', 'in_file')]),
+        (about, ds_report_about, [('out_report', 'in_file')])
     ])
 
     if anat_only:
@@ -687,6 +695,7 @@ to workflows in *QSIPrep*'s documentation]\
             ignore=ignore,
             b0_threshold=b0_threshold,
             dwi_denoise_window=dwi_denoise_window,
+            denoise_method=denoise_method,
             unringing_method=unringing_method,
             dwi_no_biascorr=dwi_no_biascorr,
             no_b0_harmonization=no_b0_harmonization,
@@ -834,6 +843,7 @@ to workflows in *QSIPrep*'s documentation]\
             confounds_name = 'inputnode.{name}_confounds'.format(name=output_wfname)
             b0_ref_name = 'inputnode.{name}_b0_ref'.format(name=output_wfname)
             cnr_name = 'inputnode.{name}_cnr'.format(name=output_wfname)
+            carpetplot_name = 'inputnode.{name}_carpetplot_data'.format(name=output_wfname)
             final_merge_wf = merging_group_workflows[concatenation_scheme[output_fname]]
             workflow.connect([
                 (dwi_finalize_wf, final_merge_wf, [
@@ -847,6 +857,7 @@ to workflows in *QSIPrep*'s documentation]\
                     ('outputnode.raw_concatenated', raw_concatenated_image_name),
                     ('outputnode.original_bvecs', original_bvec_name),
                     ('outputnode.original_files', original_bids_name),
+                    ('outputnode.carpetplot_data', carpetplot_name),
                     ('outputnode.confounds', confounds_name)])
             ])
 
